SystemOrganization addCategory: #'CS528-SetbackProject'!
SystemOrganization addCategory: #'CS528-SetbackProject-Tests'!

Object subclass: #Card
	instanceVariableNames: 'value suit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!Card commentStamp: '<historical>' prior: 0!
A Card is an instance of a playing card in a standard 52 card french deck.

Instance Variables
	suit:		symbol
	value:		symbol

suit
	- the suit of the card

value
	- the face value, or rank of the card
!

!Card class methodsFor: 'class initialization' stamp: 'TerryTata 6/29/2013 17:32'!
value: aValue suit: aSuit
((CardUtility possibleSuit includes: aSuit) & (CardUtility valueMapping includesKey: aValue))
ifTrue: [ ^self new value: aValue suit: aSuit ]
ifFalse: [ self error: 'Creation of a card is restricted to only valid suits and face values.' ]! !

!Card methodsFor: 'comparing' stamp: 'TerryTata 7/6/2013 08:45'!
< aCard
"returns true if this card has a lower face value of aCard"
^(CardUtility valueMapping at: value) < (CardUtility valueMapping at: aCard value)! !

!Card methodsFor: 'comparing' stamp: 'TerryTata 6/30/2013 13:01'!
= aCard
^(self value = aCard value) & (self suit = aCard suit)! !

!Card methodsFor: 'comparing' stamp: 'TerryTata 6/29/2013 17:32'!
> aCard
"returns true if this card has a higher face value of aCard"
^(CardUtility valueMapping at: value) > (CardUtility valueMapping at: aCard value)! !

!Card methodsFor: 'comparing' stamp: 'TerryTata 6/29/2013 10:30'!
hash
^value hash bitXor: suit hash! !

!Card methodsFor: 'printing' stamp: 'TerryTata 6/28/2013 18:37'!
printOn: aStream
(value printOn: aStream). 'of' printOn: aStream. (suit printOn: aStream)! !

!Card methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:14'!
suit
^suit copy! !

!Card methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:14'!
value
^value copy! !

!Card methodsFor: 'private' stamp: 'TerryTata 6/29/2013 10:33'!
value: aValue suit: aSuit
((value = nil) & (suit = nil))
ifTrue: [value:= aValue. suit:= aSuit.]
ifFalse: [ self error: 'Card has already been created!!' ]

! !

Object subclass: #CardUtility
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

CardUtility class
	instanceVariableNames: 'possibleSuit valueMapping possibleBids gameValues testConfig'!

!CardUtility commentStamp: '<historical>' prior: 0!
I am a simple utility class to store the possible suits and possible values of a standard playing card.!

CardUtility class
	instanceVariableNames: 'possibleSuit valueMapping possibleBids gameValues testConfig'!

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 7/21/2013 11:40'!
cokeCardMax
^9! !

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 7/21/2013 11:40'!
cokeCardMin
^3! !

!CardUtility class methodsFor: 'class initialization' stamp: 'MehrdadBiglari 7/16/2013 23:44'!
createGameValues
	gameValues := Dictionary new.
	gameValues at: #two put: 0.
	gameValues at: #three put: 0.
	gameValues at: #four put: 0.
	gameValues at: #five put: 0.
	gameValues at: #six put: 0.
	gameValues at: #seven put: 0.
	gameValues at: #eight put: 0.
	gameValues at: #nine put: 0.
	gameValues at: #ten put: 10.
	gameValues at: #jack put: 1.
	gameValues at: #queen put: 2.
	gameValues at: #king put: 3.
	gameValues at: #ace put: 4! !

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 7/1/2013 17:19'!
createPossibleBids
	possibleBids := Dictionary new.
	possibleBids at: #pass put: 1.
	possibleBids at: #two put: 2.
	possibleBids at: #three put: 3.
	possibleBids at: #four put: 4.
	possibleBids at: #five put: 5.
	! !

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 6/29/2013 15:27'!
createPossibleSuits
	possibleSuit := Bag new.
	possibleSuit add: #hearts.
	possibleSuit add: #diamonds.
	possibleSuit add: #clubs.
	possibleSuit add: #spades! !

!CardUtility class methodsFor: 'class initialization' stamp: 'MehrdadBiglari 7/16/2013 23:45'!
createValueMappings
	valueMapping := Dictionary new.
	valueMapping at: #two put: 2.
	valueMapping at: #three put: 3.
	valueMapping at: #four put: 4.
	valueMapping at: #five put: 5.
	valueMapping at: #six put: 6.
	valueMapping at: #seven put: 7.
	valueMapping at: #eight put: 8.
	valueMapping at: #nine put: 9.
	valueMapping at: #ten put: 10.
	valueMapping at: #jack put: 11.
	valueMapping at: #queen put: 12.
	valueMapping at: #king put: 13.
	valueMapping at: #ace put: 14! !

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 7/5/2013 21:55'!
gameValues
gameValues= nil
ifTrue: [ self createGameValues ].
^gameValues! !

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 7/6/2013 11:54'!
passesAroundTableDuringDeal
^2! !

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 7/1/2013 17:28'!
possibleBids
possibleBids = nil
ifTrue: [ self createPossibleBids ].
^possibleBids! !

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 6/29/2013 15:24'!
possibleSuit
possibleSuit = nil
ifTrue: [ self createPossibleSuits ].
^possibleSuit! !

!CardUtility class methodsFor: 'class initialization' stamp: 'TerryTata 7/6/2013 08:43'!
resetAllMappings
possibleSuit:= nil.
valueMapping:=nil. 
possibleBids:= nil. 
gameValues:= nil.! !

!CardUtility class methodsFor: 'class initialization' stamp: 'RameshKolavenu 7/23/2013 00:21'!
resetTestConfig
	testConfig:= nil.
! !

!CardUtility class methodsFor: 'class initialization' stamp: 'RameshKolavenu 7/23/2013 00:19'!
setTestConfig
	testConfig:= true.
! !

!CardUtility class methodsFor: 'class initialization' stamp: 'RameshKolavenu 7/23/2013 00:09'!
setbackPlayerHandSize
	testConfig isNil 
		ifTrue: 	[ ^6 ]
		ifFalse:	[ ^2 ].
	
! !

!CardUtility class methodsFor: 'class initialization' stamp: 'MehrdadBiglari 7/16/2013 23:35'!
valueMapping
valueMapping = nil
ifTrue: [ self createValueMappings ].
^valueMapping! !

Object subclass: #FrenchDeck
	instanceVariableNames: 'cards currentIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!FrenchDeck commentStamp: '<historical>' prior: 0!
I am the representation of a standard French Deck of playing cards.!

!FrenchDeck class methodsFor: 'private' stamp: 'TerryTata 6/28/2013 15:47'!
createStandardDeck
^self new createStandardDeck! !

!FrenchDeck methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:10'!
cards
^cards copy! !

!FrenchDeck methodsFor: 'private' stamp: 'TerryTata 7/19/2013 07:45'!
createStandardDeck
cards isNil
ifTrue: [ cards:= self populateFrenchDeck].
self shuffle.! !

!FrenchDeck methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/19/2013 07:49'!
currentIndex
^currentIndex copy! !

!FrenchDeck methodsFor: 'dealing' stamp: 'TerryTata 6/28/2013 18:53'!
dealNext
currentIndex:= currentIndex + 1.
^cards at: currentIndex ifAbsent: [ self error: 'The deck has run out of cards!!' ]! !

!FrenchDeck methodsFor: 'private' stamp: 'TerryTata 6/29/2013 17:48'!
populateFrenchDeck
|deck index|
deck:= Array new: 52.
index:= 1.
CardUtility valueMapping keysDo: 
[ :value | CardUtility possibleSuit do: 
	[ :suit | deck at: index put: (Card value: value suit: suit). index:= index + 1.]].
^deck! !

!FrenchDeck methodsFor: 'printing' stamp: 'TerryTata 6/28/2013 18:28'!
printOn: aStream
cards do: [ :card | card printOn: aStream ]! !

!FrenchDeck methodsFor: 'shuffling' stamp: 'TerryTata 6/28/2013 19:42'!
shuffle
cards shuffle.
currentIndex:= 0. ! !

Object subclass: #PlayerConfigStrategy
	instanceVariableNames: 'numberOfPlayers onTeams'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

PlayerConfigStrategy class
	instanceVariableNames: 'instance'!

!PlayerConfigStrategy commentStamp: '<historical>' prior: 0!
A PlayerConfigStrategy is xxxxxxxxx.

Instance Variables
	numberOfPlayers:		<Object>

numberOfPlayers
	- xxxxx
!

PlayerConfigStrategy class
	instanceVariableNames: 'instance'!

PlayerConfigStrategy subclass: #FourPlayerNoTeamsStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!FourPlayerNoTeamsStrategy commentStamp: '<historical>' prior: 0!
A FourPlayerNoTeamsStrategy is xxxxxxxxx.

Instance Variables
!

!FourPlayerNoTeamsStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/11/2013 10:39'!
determinePlayerLimit
numberOfPlayers:= 4.
onTeams:= false.! !

PlayerConfigStrategy subclass: #FourPlayerTeamsStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!FourPlayerTeamsStrategy commentStamp: '<historical>' prior: 0!
A FourPlayerTeamsStrategy is xxxxxxxxx.

Instance Variables
!

!FourPlayerTeamsStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/11/2013 10:34'!
determinePlayerLimit
numberOfPlayers:= 4.
onTeams:= true.! !

!PlayerConfigStrategy class methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/11/2013 10:36'!
new
    instance isNil ifTrue: [instance := super new. instance determinePlayerLimit.].
    ^ instance! !

!PlayerConfigStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 17:19'!
determinePlayerLimit
^self subclassResponsibility ! !

!PlayerConfigStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 18:21'!
onTeams
^onTeams.! !

!PlayerConfigStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 18:08'!
players
^numberOfPlayers copy! !

PlayerConfigStrategy subclass: #ThreePlayerStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!ThreePlayerStrategy commentStamp: '<historical>' prior: 0!
A ThreePlayerStrategy is xxxxxxxxx.

Instance Variables
!

!ThreePlayerStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 18:21'!
determinePlayerLimit
numberOfPlayers:= 3.
onTeams:= false.! !

Object subclass: #SetbackGame
	instanceVariableNames: 'players playerConfigStrategy winnerStrategy phaseOrder started'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackGame commentStamp: '<historical>' prior: 0!
A SetbackGame is xxxxxxxxx.

Instance Variables
	playerConfigStrategy:		<Object>
	players:		<Object>
	started:		<Object>
	winnerStrategy:		<Object>

playerConfigStrategy
	- xxxxx

players
	- xxxxx

started
	- xxxxx

winnerStrategy
	- xxxxx
!

!SetbackGame class methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/11/2013 11:55'!
config: aConfig winner: aWinner order: aOrder
^self new config: aConfig winner: aWinner order: aOrder.! !

!SetbackGame class methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:39'!
setUpStandardGame
^self new playerConfigStrategy: (ThreePlayerStrategy new);
winnerStrategy: (StandardWinStrategy new); phaseOrder: (SetbackPhaseOrder new).! !

!SetbackGame class methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:39'!
setUpStandardTeamGame
^self new playerConfigStrategy: (FourPlayerTeamsStrategy new);
winnerStrategy: (StandardWinStrategy new); phaseOrder: (SetbackPhaseOrder new).! !

!SetbackGame methodsFor: 'adding' stamp: 'TerryTata 6/30/2013 18:07'!
addPlayer: aPlayer
(players size = playerConfigStrategy players)
ifTrue: [ self error: 'You already have the maximum alloted players under your current configuration!!' ].
started 
ifTrue: [ self error: 'You cannot add new players after the game has started!!' ].
players add: aPlayer.! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'TerryTata 7/11/2013 11:56'!
config: aConfig winner: aWinner order: aOrder
self playerConfigStrategy: aConfig.
self winnerStrategy: aWinner.
self phaseOrder: aOrder.! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'TerryTata 6/30/2013 20:38'!
initialize
players:= LinkedList new.
started:= false.! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'TerryTata 7/7/2013 17:01'!
phaseOrder: somePhases
started
ifTrue: [ self error: 'The game has already started. You may not alter the order of phases!!' ].
phaseOrder:= somePhases.
! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'MehrdadBiglari 7/22/2013 22:51'!
phaseOrderName
	^(phaseOrder class) name.! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'TerryTata 6/30/2013 18:26'!
playerConfigStrategy: aConfig
started
ifTrue: [ self error: 'The game has already started. You may not alter the configuration!!' ].
playerConfigStrategy:= aConfig.
! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'TerryTata 11/29/1999 16:40'!
startGame
(players size = playerConfigStrategy players)
ifFalse: [ self error: 'You do not have the proper number of players to start a game!!' ].
started
ifTrue: [ self error: 'The game has already started!! Start a new hand instead!!' ].
players do: [ :player | player resetForNewGame ].
started:= true.
^self startNewHand.! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'MehrdadBiglari 7/19/2013 21:20'!
startNewHand
started
ifFalse: [ self error: 'You may not start a new hand if the game hasn''t started!!' ].
players do: [ :player | player resetForNewGameHand ].

^SetbackGameHand players: players playerConfig: playerConfigStrategy winnerStrat: winnerStrategy phaseOrder: phaseOrder.
! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'TerryTata 7/2/2013 19:12'!
started
^started copy! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'TerryTata 6/30/2013 18:27'!
winnerStrategy: aWinner
started
ifTrue: [ self error: 'The game has already started. You may not alter the configuration!!' ].
winnerStrategy:= aWinner.
! !

!SetbackGame methodsFor: 'initialize-release' stamp: 'MehrdadBiglari 7/22/2013 22:52'!
winnerStrategyName
	^(winnerStrategy class) name.
! !

Object subclass: #SetbackGameHand
	instanceVariableNames: 'players playerConfigStrategy winnerStrategy deck trump bidder dealer currentBid currentPhase phaseOrder winner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackGameHand commentStamp: '<historical>' prior: 0!
A SetbackGameHand is xxxxxxxxx.

Instance Variables
	bidder:		<Object>
	currentBid:		<Object>
	dealer:		<Object>
	deck:		<Object>
	playerLead:		<Object>
	players:		<Object>
	trump:		<Object>

bidder
	- xxxxx

currentBid
	- xxxxx

dealer
	- xxxxx

deck
	- xxxxx

playerLead
	- xxxxx

players
	- xxxxx

trump
	- xxxxx
!

!SetbackGameHand class methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 17:03'!
players: players playerConfig: aConfig winnerStrat: aWinner phaseOrder: aPhaseOrder
^self new players: players playerConfig: aConfig winnerStrat: aWinner phaseOrder: aPhaseOrder! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/7/2013 16:11'!
bidder
^bidder! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/7/2013 16:11'!
currentBid
^currentBid! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/7/2013 16:11'!
currentPhase
^currentPhase! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/7/2013 16:12'!
dealer
^dealer! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/7/2013 16:16'!
deck
^deck! !

!SetbackGameHand methodsFor: 'phases' stamp: 'TerryTata 7/2/2013 17:25'!
endBidPhase: aBid by: aBidder
bidder:= aBidder.
currentBid:= aBid! !

!SetbackGameHand methodsFor: 'phases' stamp: 'MehrdadBiglari 7/19/2013 23:12'!
endScoringPhase
	winner:= (winnerStrategy doWeHaveAWinner: players bidder: bidder playerConfig: playerConfigStrategy).
	winner isNil
	ifFalse: [
		players do: [ :player | player resetForNewGame ]. 
		playerConfigStrategy onTeams 
		ifTrue: [Transcript show: 'Congratulations to our game winners: ', (winner listNames), '!!' ]
		ifFalse: [ Transcript show: 'Congratulations to our game winner: ', winner name, ' !!' ]]
	! !

!SetbackGameHand methodsFor: 'phases' stamp: 'TerryTata 7/2/2013 21:34'!
endTrickPhase: trumpSuit
trump:= trumpSuit.! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/7/2013 18:09'!
playerConfigStrategy
^playerConfigStrategy ! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/2/2013 19:43'!
players
^players copy! !

!SetbackGameHand methodsFor: 'initialize-release' stamp: 'MehrdadBiglari 7/19/2013 21:20'!
players: somePlayers playerConfig: aConfig winnerStrat: aWinner phaseOrder: aPhaseOrder
players = nil
ifFalse: [ self error: 'The hand has already started!! You may not change the players!!' ].
players:= somePlayers.
deck:= FrenchDeck createStandardDeck.
trump:= nil.
bidder:= nil.
currentBid:= nil.
currentPhase:= nil.
"set the order in which bidding will occur, as well as the dealer"
dealer:= players removeFirst.
players add: dealer.
playerConfigStrategy:= aConfig. 
winnerStrategy:= aWinner.
phaseOrder:= aPhaseOrder.
phaseOrder gameHand: self.! !

!SetbackGameHand methodsFor: 'phases' stamp: 'TerryTata 7/7/2013 19:22'!
startNextPhase
currentPhase:= phaseOrder advancePhase.
currentPhase isNil
ifFalse: [ currentPhase startProcessing. ]! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/7/2013 16:12'!
trump
^trump! !

!SetbackGameHand methodsFor: 'phases' stamp: 'MehrdadBiglari 7/19/2013 23:12'!
winner
	^winner.! !

!SetbackGameHand methodsFor: 'accessing' stamp: 'TerryTata 7/12/2013 13:33'!
winnerStrategy
^winnerStrategy ! !

Object subclass: #SetbackGamePhase
	instanceVariableNames: 'players playerConfigStrategy winnerStrategy'
	classVariableNames: 'gameHand'
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackGamePhase commentStamp: '<historical>' prior: 0!
A SetbackGamePhase is xxxxxxxxx.

Instance Variables
	gameHand:		<Object>
	players:		<Object>

gameHand
	- xxxxx

players
	- xxxxx
!

SetbackGamePhase subclass: #BidPhase
	instanceVariableNames: 'currentBidder winningBidder currentBid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!BidPhase commentStamp: '<historical>' prior: 0!
A BidPhase is xxxxxxxxx.

Instance Variables
!

!BidPhase methodsFor: 'initialize-release' stamp: 'TerryTata 7/14/2013 09:42'!
advancePhase: phaseOrder
phaseOrder advanceToTrickPhase.! !

!BidPhase methodsFor: 'initialize-release' stamp: 'MehrdadBiglari 7/17/2013 21:02'!
currentBidder
^currentBidder! !

!BidPhase methodsFor: 'initialize-release' stamp: 'TerryTata 7/14/2013 09:42'!
reportStatus
players size = 0
ifTrue: [ gameHand endBidPhase: currentBid by: winningBidder.]! !

BidPhase subclass: #StandardBidPhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!StandardBidPhase commentStamp: '<historical>' prior: 0!
A BidPhase is xxxxxxxxx.

Instance Variables
	currentBid:		<Object>
	currentBidder:		<Object>
	winningBidder:		<Object>

currentBid
	- xxxxx

currentBidder
	- xxxxx

winningBidder
	- xxxxx
!

!StandardBidPhase methodsFor: 'initialize-release' stamp: 'TerryTata 7/23/2013 19:12'!
reportTo: aGameHand
super reportTo: aGameHand.! !

!StandardBidPhase methodsFor: 'initialize-release' stamp: 'TerryTata 7/23/2013 19:12'!
startProcessing
currentBid:= #pass.
currentBidder:= nil.
winningBidder:= nil.
self waitForBid.! !

!StandardBidPhase methodsFor: 'initialize-release' stamp: 'TerryTata 7/7/2013 19:32'!
update: aSymbol
((currentBid = #pass) & (players size = 0) & (aSymbol = #pass))
"dealer must bid if no one else has"
ifTrue: [ winningBidder:= currentBidder. currentBid:= #two. ]
ifFalse:[
aSymbol = #pass
ifFalse: [
	"check to see if the bid that was made was larger than the last bid"  
	(CardUtility possibleBids at: aSymbol) > (CardUtility possibleBids at: currentBid)
	ifTrue: [ winningBidder:= currentBidder. currentBid:= aSymbol. ].
	"if the dealer is bidding, s/he can steal the bid for the same bid amount"
	(((CardUtility possibleBids at: aSymbol) = (CardUtility possibleBids at: currentBid)) & (players size =  0))
	ifTrue: [ winningBidder:= currentBidder. currentBid:= aSymbol.]]].

currentBidder removeDependent: self.

players size = 0
ifTrue: [  self reportStatus.]
ifFalse: [ self waitForBid. ]! !

!StandardBidPhase methodsFor: 'initialize-release' stamp: 'TerryTata 7/2/2013 17:37'!
waitForBid
players size > 0
ifTrue: [  
currentBidder:= players removeFirst.
currentBidder addDependent: self.
]
ifFalse: [ self error: 'All players have had their turn to bid!!' ]! !

SetbackGamePhase subclass: #DealPhase
	instanceVariableNames: 'deck'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!DealPhase commentStamp: '<historical>' prior: 0!
A DealPhase is xxxxxxxxx.

Instance Variables
!

!DealPhase methodsFor: 'phases' stamp: 'TerryTata 7/14/2013 10:07'!
advancePhase: phaseOrder
phaseOrder advanceToBidPhase.! !

!DealPhase methodsFor: 'phases' stamp: 'TerryTata 7/14/2013 10:27'!
reportStatus
"do nothing - players are dealt cards directly"! !

DealPhase subclass: #StandardDealPhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!StandardDealPhase commentStamp: '<historical>' prior: 0!
A DealPhase is xxxxxxxxx.

Instance Variables
	deck:		<Object>

deck
	- xxxxx
!

!StandardDealPhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/6/2013 22:14'!
dealPlayer: aPlayer
	"deal a player 3 cards each pass"

	1 to: CardUtility setbackPlayerHandSize / CardUtility passesAroundTableDuringDeal do: [ :i | aPlayer beDealtCard: deck dealNext ]! !

!StandardDealPhase methodsFor: 'phases' stamp: 'TerryTata 7/23/2013 19:13'!
reportTo: aHand
super reportTo: aHand.! !

!StandardDealPhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/6/2013 22:14'!
startPass
"two passes around the table"
1 to: (CardUtility passesAroundTableDuringDeal) do: [:i | players do: [ :player | self dealPlayer: player. ]]! !

!StandardDealPhase methodsFor: 'phases' stamp: 'TerryTata 7/23/2013 19:14'!
startProcessing
deck:= gameHand deck.
self startPass.! !

SetbackGamePhase subclass: #ScorePhase
	instanceVariableNames: 'trump bidder bid pointsMapping'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!ScorePhase commentStamp: '<historical>' prior: 0!
A ScorePhase is xxxxxxxxx.

Instance Variables
!

ScorePhase subclass: #NoGarbagePointsScorePhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!NoGarbagePointsScorePhase commentStamp: '<historical>' prior: 0!
A GarbagePointsScorePhase is xxxxxxxxx.

Instance Variables
!

!NoGarbagePointsScorePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 09:37'!
assignBidderPoints: bidderPoints
	bidderPoints >= (CardUtility possibleBids at: bid)
		ifTrue: [ bidder addToPointsEarned: (CardUtility possibleBids at: bid) ]
		ifFalse: [ bidder addToPointsEarned: (CardUtility possibleBids at: bid) * -1 ].! !

!NoGarbagePointsScorePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 09:34'!
createPointsMapping
super createPointsMapping: false.! !

!ScorePhase methodsFor: 'updating' stamp: 'TerryTata 7/14/2013 10:11'!
advancePhase: phaseOrder
phaseOrder advanceBeyondScorePhase.! !

!ScorePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 09:35'!
assignBidderPoints: bidderPoints
^self subclassResponsibility ! !

!ScorePhase methodsFor: 'updating' stamp: 'TerryTata 7/21/2013 09:37'!
beginScoring
	| bidderValue |
	bidderValue := 0.
	players do: [ :player | self calculatePoints: player ].
	pointsMapping do: [ :point | point accountForTeams: players ].
	pointsMapping do: [ :point | bidderValue := bidderValue + (point awardPoint: bidder) ].
	self assignBidderPoints: bidderValue.
	self reportStatus! !

!ScorePhase methodsFor: 'updating' stamp: 'TerryTata 7/21/2013 09:27'!
calculatePoints: aPlayer
	aPlayer cardsWon
		do: [ :card | pointsMapping do: [ :point | 
				point doesCardWinPoint: card from: aPlayer. ]. ].! !

!ScorePhase methodsFor: 'updating' stamp: 'TerryTata 7/21/2013 09:33'!
createPointsMapping
^self subclassResponsibility! !

!ScorePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 09:32'!
createPointsMapping: junkPoints
pointsMapping := LinkedList new.
	pointsMapping
		add:
			(HighPoint
				winningPlayer: nil
				winningValue: nil
				trump: trump
				junkPoints: junkPoints).
	pointsMapping
		add:
			(LowPoint
				winningPlayer: nil
				winningValue: nil
				trump: trump
				junkPoints: junkPoints).
	pointsMapping
		add:
			(GamePoint
				winningPlayer: nil
				winningValue: nil
				trump: trump
				junkPoints: junkPoints).
	pointsMapping
		add:
			(JackPoint
				winningPlayer: nil
				winningValue: nil
				trump: trump
				junkPoints: junkPoints)! !

!ScorePhase methodsFor: 'updating' stamp: 'TerryTata 7/14/2013 10:11'!
reportStatus 
gameHand endScoringPhase.! !

!ScorePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:14'!
reportTo: aGameHand
	super reportTo: aGameHand.! !

!ScorePhase methodsFor: 'updating' stamp: 'TerryTata 7/23/2013 19:14'!
startProcessing
	trump := gameHand trump.
	bidder := gameHand bidder.
	bid := gameHand currentBid.	"four points to hand out"
	self createPointsMapping.
self beginScoring.! !

ScorePhase subclass: #StandardScorePhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!StandardScorePhase commentStamp: '<historical>' prior: 0!
A ScorePhase is xxxxxxxxx.

Instance Variables
	currentTrick:		<Object>
	currentTrickCardsThrown:		<Object>
	nextPlayerToThrow:		<Object>
	playerConfigStrategy:		<Object>
	remainingTricks:		<Object>
	trump:		<Object>

currentTrick
	- xxxxx

currentTrickCardsThrown
	- xxxxx

nextPlayerToThrow
	- xxxxx

playerConfigStrategy
	- xxxxx

remainingTricks
	- xxxxx

trump
	- xxxxx
!

!StandardScorePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 09:37'!
assignBidderPoints: bidderPoints
	bidderPoints >= (CardUtility possibleBids at: bid)
		ifTrue: [ bidder addToPointsEarned: bidderPoints ]
		ifFalse: [ bidder addToPointsEarned: (CardUtility possibleBids at: bid) * -1 ].! !

!StandardScorePhase methodsFor: 'updating' stamp: 'TerryTata 7/21/2013 09:43'!
createPointsMapping
super createPointsMapping: true.! !

!SetbackGamePhase class methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 16:19'!
reportTo: gameHand
^self new reportTo: gameHand.! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/14/2013 09:16'!
advancePhase: phaseOrder
^self subclassResponsibility! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'MehrdadBiglari 7/17/2013 22:20'!
currentBidder
	^nil.! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'MehrdadBiglari 7/18/2013 22:08'!
currentTrickCardsThrown
	^nil.! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'MehrdadBiglari 7/17/2013 21:49'!
nextPlayerToThrow
	^nil.! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'MehrdadBiglari 7/18/2013 22:38'!
remainingTricks
^1! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 17:14'!
reportStatus
^self subclassResponsibility! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:34'!
reportTo: aHand
gameHand:= aHand.
playerConfigStrategy:= aHand playerConfigStrategy.
winnerStrategy:= aHand winnerStrategy.
players:= aHand players.! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'MehrdadBiglari 7/18/2013 22:24'!
shouldDisplayTrump
^false.! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 16:07'!
startProcessing
^self subclassResponsibility! !

!SetbackGamePhase methodsFor: 'as yet unclassified' stamp: 'MehrdadBiglari 7/18/2013 21:26'!
trump
	^nil.! !

SetbackGamePhase subclass: #TrickPhase
	instanceVariableNames: 'currentTrickCardsThrown nextPlayerToThrow currentTrick trump remainingTricks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!TrickPhase commentStamp: '<historical>' prior: 0!
A TrickPhase is xxxxxxxxx.

Instance Variables
!

TrickPhase subclass: #StandardTrickPhase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!StandardTrickPhase commentStamp: '<historical>' prior: 0!
A TrickPhase is xxxxxxxxx.

Instance Variables
	currentTrickCardsThrown:		<Object>
	nextPlayerToThrow:		<Object>
	remainingTricks:		<Object>

currentTrickCardsThrown
	- xxxxx

nextPlayerToThrow
	- xxxxx

remainingTricks
	- xxxxx
!

!StandardTrickPhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:14'!
reportTo: aGameHand
super reportTo: aGameHand.! !

!StandardTrickPhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/2/2013 21:50'!
setPlayerOrder
"sets the traversal order for the current trick about to be played"
|currentHead|
currentHead:= players removeFirst.
currentHead = nextPlayerToThrow
ifTrue: [ players addFirst: currentHead ]
ifFalse: [ players addLast: currentHead. self setPlayerOrder. ]! !

!StandardTrickPhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/6/2013 21:13'!
startNewTrick
nextPlayerToThrow:= currentTrickCardsThrown at: (currentTrick winningCard).
nextPlayerToThrow winTrick: currentTrick cards.
currentTrickCardsThrown:= currentTrickCardsThrown removeAll.
currentTrick:= nil.
remainingTricks:= remainingTricks - 1.
self setPlayerOrder.! !

!StandardTrickPhase methodsFor: 'updating' stamp: 'TerryTata 7/23/2013 19:23'!
startProcessing
nextPlayerToThrow:= gameHand bidder.
currentTrickCardsThrown := Dictionary new.
remainingTricks:= CardUtility setbackPlayerHandSize.
trump:= nil.
currentTrick:= nil.
self setPlayerOrder.
self waitForCard.! !

!StandardTrickPhase methodsFor: 'updating' stamp: 'TerryTata 7/12/2013 15:24'!
update: aCard
"just play the card"
currentTrick ~= nil
ifTrue: [ currentTrick playCard: aCard inHand: (nextPlayerToThrow hand) ].
"first trick of a new game. we're establishing trump!!"
((currentTrick = nil) & (remainingTricks = CardUtility setbackPlayerHandSize))
ifTrue: [ currentTrick:= SetbackTrick playFirstCardOfNewGame: aCard inHand: (nextPlayerToThrow hand). trump:= aCard suit.].
"starting subsequent tricks of  a game"
((currentTrick = nil) & (remainingTricks < CardUtility setbackPlayerHandSize))
ifTrue: [ currentTrick:= SetbackTrick leadCard: aCard inHand: (nextPlayerToThrow hand).].

currentTrickCardsThrown at: aCard put: nextPlayerToThrow.

nextPlayerToThrow removeDependent: self.

currentTrickCardsThrown size >= playerConfigStrategy players
ifTrue: [ self startNewTrick. ].
self waitForCard.! !

!StandardTrickPhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 17:15'!
waitForCard
remainingTricks > 0
ifTrue: [ 
	nextPlayerToThrow:= players removeFirst.
	nextPlayerToThrow addDependent: self.
	players add: nextPlayerToThrow.
]
ifFalse: [ self reportStatus. ]! !

!TrickPhase methodsFor: 'updating' stamp: 'TerryTata 7/14/2013 10:15'!
advancePhase: phaseOrder
phaseOrder advanceToScorePhase.! !

!TrickPhase methodsFor: 'updating' stamp: 'MehrdadBiglari 7/17/2013 21:43'!
currentTrickCardsThrown
^currentTrickCardsThrown! !

!TrickPhase methodsFor: 'updating' stamp: 'MehrdadBiglari 7/17/2013 21:43'!
nextPlayerToThrow
^nextPlayerToThrow! !

!TrickPhase methodsFor: 'updating' stamp: 'MehrdadBiglari 7/18/2013 22:38'!
remainingTricks
^remainingTricks! !

!TrickPhase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/14/2013 10:15'!
reportStatus
remainingTricks = 0
ifTrue: [ gameHand endTrickPhase: trump]! !

!TrickPhase methodsFor: 'updating' stamp: 'MehrdadBiglari 7/18/2013 22:24'!
shouldDisplayTrump
^true.! !

!TrickPhase methodsFor: 'updating' stamp: 'MehrdadBiglari 7/17/2013 21:44'!
trump
^trump! !

Object subclass: #SetbackPhaseOrder
	instanceVariableNames: 'dealPhase bidPhase trickPhase scorePhase currentPhase gameHand'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackPhaseOrder commentStamp: '<historical>' prior: 0!
A SetbackPhaseOrder is xxxxxxxxx.

Instance Variables
!

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 10:33'!
advanceBeyondScorePhase
currentPhase:= nil.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/14/2013 09:16'!
advancePhase
currentPhase isNil
ifTrue: [ self advanceToDealPhase ]
ifFalse: [ currentPhase advancePhase: self ].
^currentPhase! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:34'!
advanceToBidPhase
currentPhase:= (Smalltalk at: bidPhase) reportTo: gameHand.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:35'!
advanceToDealPhase
currentPhase:= (Smalltalk at: dealPhase) reportTo: gameHand.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:35'!
advanceToScorePhase
currentPhase:= (Smalltalk at: scorePhase) reportTo: gameHand.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:35'!
advanceToTrickPhase
currentPhase:= (Smalltalk at: trickPhase) reportTo: gameHand.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:29'!
bidPhase: aBidPhase
bidPhase:= aBidPhase.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:29'!
dealPhase: aDealPhase
dealPhase:= aDealPhase.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:34'!
gameHand: aHand
gameHand:= aHand.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:35'!
initialize
currentPhase:= nil.
dealPhase:= #StandardDealPhase.
bidPhase:= #StandardBidPhase. 
trickPhase:= #StandardTrickPhase.
scorePhase:= #StandardScorePhase.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:44'!
scorePhase: aScorePhase
scorePhase:= aScorePhase.! !

!SetbackPhaseOrder methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:44'!
trickPhase: aTrickPhase
trickPhase:= aTrickPhase.! !

Object subclass: #SetbackPlayerHand
	instanceVariableNames: 'cards'
	classVariableNames: 'capacity'
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackPlayerHand commentStamp: '<historical>' prior: 0!
I am the representation of a standard setback hand. I contain a maximum of six cards and am owned by exactly one player.!

!SetbackPlayerHand class methodsFor: 'class initialization' stamp: 'TerryTata 7/5/2013 18:50'!
addCard: aCard
capacity:= CardUtility setbackPlayerHandSize.
^self new addCard: aCard.! !

!SetbackPlayerHand methodsFor: 'adding' stamp: 'TerryTata 7/5/2013 18:45'!
addCard: aCard
cards size >= capacity
ifTrue: [ self error: 'Cannot add any more cards to hand!!' ]
ifFalse: [ cards add: aCard. ]
! !

!SetbackPlayerHand methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:11'!
capacity
^capacity copy! !

!SetbackPlayerHand methodsFor: 'accessing' stamp: 'MehrdadBiglari 7/16/2013 23:37'!
cards
^cards copy! !

!SetbackPlayerHand methodsFor: 'testing' stamp: 'TerryTata 6/29/2013 19:18'!
includes: aCard
^cards includes: aCard! !

!SetbackPlayerHand methodsFor: 'initialize-release' stamp: 'TerryTata 6/28/2013 20:10'!
initialize
cards:= Set new.! !

!SetbackPlayerHand methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 11:42'!
isCokeHand
	^ (cards
		select: [ :card | 
			(CardUtility valueMapping at: card value) > CardUtility cokeCardMax
				| ((CardUtility valueMapping at: card value) < CardUtility cokeCardMin) ]) isEmpty! !

!SetbackPlayerHand methodsFor: 'adding' stamp: 'TerryTata 7/2/2013 20:22'!
removeCard: aCard
cards size = 0
ifTrue: [ self error: 'Hand is empty!!' ].
(cards includes: aCard)
ifTrue: [ cards remove: aCard.]
ifFalse: [ self error: 'Card does not exist in hand!!'].! !

!SetbackPlayerHand methodsFor: 'testing' stamp: 'TerryTata 6/29/2013 19:42'!
suitExists: aSuit
^(cards select: [ :each | each suit = aSuit]) isEmpty not! !

Object subclass: #SetbackPlayerTeam
	instanceVariableNames: 'pointsEarned players'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackPlayerTeam commentStamp: '<historical>' prior: 0!
A SetbackPlayerTeam is xxxxxxxxx.

Instance Variables
!

!SetbackPlayerTeam methodsFor: 'adding' stamp: 'TerryTata 7/11/2013 11:09'!
addPlayer: aPlayer
players add: aPlayer! !

!SetbackPlayerTeam methodsFor: 'accessing' stamp: 'TerryTata 7/13/2013 08:44'!
gamePointsEarned
^players inject: 0 into: [ :sum :player | sum + player gamePointsWon ]! !

!SetbackPlayerTeam methodsFor: 'actions' stamp: 'TerryTata 7/12/2013 14:29'!
incrementGamesWon
players do: [ :player | player incrementGamesWon  ]! !

!SetbackPlayerTeam methodsFor: 'initialize-release' stamp: 'TerryTata 7/11/2013 11:10'!
initialize
pointsEarned:= 0.
players:= LinkedList new.! !

!SetbackPlayerTeam methodsFor: 'accessing' stamp: 'TerryTata 7/13/2013 10:06'!
listNames
^players inject: '' into: [ :string :player | string, player name, ' ' ].! !

!SetbackPlayerTeam methodsFor: 'accessing' stamp: 'TerryTata 7/11/2013 11:10'!
players
^players copy! !

!SetbackPlayerTeam methodsFor: 'accessing' stamp: 'TerryTata 7/13/2013 08:36'!
pointsEarned
^players inject: 0 into: [ :sum :player | sum + player pointsEarned ]! !

!SetbackPlayerTeam methodsFor: 'adding' stamp: 'TerryTata 11/29/1999 16:56'!
removePlayer: aPlayer
 players remove: aPlayer.! !

Object subclass: #SetbackPoint
	instanceVariableNames: 'winningPlayer winningValue trump junkPoints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackPoint commentStamp: '<historical>' prior: 0!
A SetbackPoint is xxxxxxxxx.

Instance Variables
	winningPlayer:		<Object>
	winningValue:		<Object>

winningPlayer
	- xxxxx

winningValue
	- xxxxx
!

SetbackPoint subclass: #GamePoint
	instanceVariableNames: 'duplicateWinner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!GamePoint commentStamp: '<historical>' prior: 0!
A GamePoint is xxxxxxxxx.

Instance Variables
	duplicateWinner:		<Object>

duplicateWinner
	- xxxxx
!

!GamePoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 09:28'!
accountForTeams: players
	| team1 team2 |
	team1 := players first currentTeam.
	team2 := players last currentTeam.
	team1 isNil | team2 isNil
		ifFalse: [
			duplicateWinner := false.
			team1 gamePointsEarned = team2 gamePointsEarned
				ifTrue: [ duplicateWinner := true ].
			team1 gamePointsEarned > team2 gamePointsEarned
				ifTrue: [ winningPlayer := players first ]
				ifFalse: [ winningPlayer := players last ] ].! !

!GamePoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 09:05'!
awardPoint: bidder
	duplicateWinner
		ifTrue: [ ^ 0 ]
		ifFalse: [ ^super awardPoint: bidder ]! !

!GamePoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 09:27'!
doesCardWinPoint: aCard from: aPlayer
	| gamePoints |
	aPlayer addToGamePoints: (CardUtility gameValues at: aCard value).
	gamePoints := aPlayer gamePointsWon.
	winningValue isNil
		ifTrue: [ 
			winningPlayer := aPlayer.
			winningValue := gamePoints.
			duplicateWinner := false ]
		ifFalse: [ 
			gamePoints = winningValue
				ifTrue: [ duplicateWinner := true ].
			gamePoints > winningValue
				ifTrue: [ 
					winningPlayer := aPlayer.
					winningValue := gamePoints.
					duplicateWinner := false ] ]! !

SetbackPoint subclass: #HighPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!HighPoint commentStamp: '<historical>' prior: 0!
A HighPoint is xxxxxxxxx.

Instance Variables
!

!HighPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/6/2013 16:13'!
doesCardWinPoint: aCard from: aPlayer
super compareCard: aCard from: aPlayer using: #>
! !

SetbackPoint subclass: #JackPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!JackPoint commentStamp: '<historical>' prior: 0!
A JackPoint is xxxxxxxxx.

Instance Variables
!

!JackPoint methodsFor: 'as yet unclassified' stamp: 'MehrdadBiglari 7/19/2013 21:05'!
doesCardWinPoint: aCard from: aPlayer
((winningValue isNil) & (aCard value = #jack) & (aCard suit = trump))
ifTrue: [ winningPlayer:= aPlayer. winningValue:= aCard. ]! !

SetbackPoint subclass: #LowPoint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!LowPoint commentStamp: '<historical>' prior: 0!
A LowPoint is xxxxxxxxx.

Instance Variables
!

!LowPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/6/2013 16:14'!
doesCardWinPoint: aCard from: aPlayer
super compareCard: aCard from: aPlayer using: #<! !

!SetbackPoint class methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 09:04'!
winningPlayer: aPlayer winningValue: aValue trump: aSuit junkPoints: anObject
	^ self new
		winningPlayer: aPlayer
		winningValue: aValue
		trump: aSuit
		junkPoints: anObject! !

!SetbackPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 14:00'!
accountForTeams: somePlayers
"do nothing - let subclasses override if a point is determined differently for teams."! !

!SetbackPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 11:56'!
awardPoint: bidder
	| bidderPoints |
	bidderPoints := 0.
	winningValue isNil
		ifFalse: [ 
			winningPlayer = bidder
				ifTrue: [ bidderPoints := 1 ]
				ifFalse: [ 
					bidder currentTeam isNil
						ifTrue: [ bidderPoints := self calculateNoTeams ]
						ifFalse: [ bidderPoints := self calculateWithTeams: bidder ] ] ].
	^ bidderPoints! !

!SetbackPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 09:42'!
calculateNoTeams
	junkPoints
		ifTrue: [ winningPlayer addToPointsEarned: 1 ].
	^ 0! !

!SetbackPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 11:55'!
calculateWithTeams: bidder
	winningPlayer currentTeam = bidder currentTeam
		ifTrue: [ ^ 1 ].
	^self calculateNoTeams.! !

!SetbackPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 15:41'!
compareCard: aCard from: aPlayer using: aComparator
aCard suit = trump
ifTrue: [ winningValue isNil
ifTrue: [  winningPlayer:= aPlayer. winningValue:= aCard ]
ifFalse: [ (aCard perform: aComparator with: winningValue) ifTrue: [  winningPlayer:= aPlayer. winningValue:= aCard  ]]. ]! !

!SetbackPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/6/2013 10:01'!
doesCardWinPoint: aCard from: aPlayer
^self subclassResponsibility ! !

!SetbackPoint methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 09:05'!
winningPlayer: aPlayer winningValue: aValue trump: aSuit junkPoints: anObject
trump:= aSuit.
winningPlayer:= aPlayer.
winningValue:= aValue.
junkPoints:= anObject! !

Object subclass: #SetbackTrick
	instanceVariableNames: 'cards leadCard winningCard'
	classVariableNames: 'trumpSuit'
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackTrick commentStamp: '<historical>' prior: 0!
I am an individual trick of a setback hand. I contain all logic for determining the winning card of a trick based on the current trump suit and the leading suit.!

!SetbackTrick class methodsFor: 'initialize-release' stamp: 'TerryTata 6/29/2013 19:52'!
leadCard: aCard inHand: aHand
^self new leadCard: aCard inHand: aHand! !

!SetbackTrick class methodsFor: 'initialize-release' stamp: 'TerryTata 6/29/2013 19:53'!
playFirstCardOfNewGame: aCard inHand: aHand
self trump: aCard suit.
^self leadCard: aCard inHand: aHand.! !

!SetbackTrick class methodsFor: 'initialize-release' stamp: 'TerryTata 6/29/2013 17:32'!
trump: aSuit
(CardUtility possibleSuit includes: aSuit)
ifTrue: [ trumpSuit:= aSuit. ]
ifFalse: [ self error: 'An invalid suit was selected as trump' ]! !

!SetbackTrick methodsFor: 'validate' stamp: 'TerryTata 6/29/2013 20:04'!
blessCard: aCard thrownBy: aHand
(aHand includes: aCard)
ifFalse: [ self error: 'You may only play cards that are in your hand!!' ].! !

!SetbackTrick methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:15'!
cards
^cards copy! !

!SetbackTrick methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:15'!
leadCard
^leadCard copy! !

!SetbackTrick methodsFor: 'initialize-release' stamp: 'TerryTata 6/29/2013 20:05'!
leadCard: aCard inHand: aHand
self blessCard: aCard thrownBy: aHand.
"this method should only be called once (when the first card is thrown of a new trick)"
(cards isNil) & (leadCard isNil)
ifTrue: [ cards:= Set new. leadCard:= aCard. winningCard:= leadCard.]
ifFalse: [ self error: 'The lead card has already been thrown!!'].

self playCard: aCard inHand: aHand.! !

!SetbackTrick methodsFor: 'initialize-release' stamp: 'TerryTata 6/29/2013 20:05'!
playCard: aCard inHand: aHand
self blessCard: aCard thrownBy: aHand.
"plays a card in the current trick. Validates the hand to ensure no cheating occured"
aCard suit = trumpSuit
ifTrue: [ self playTrumpCard: aCard. ]
ifFalse: [ (((aHand suitExists: (leadCard suit)) not) | (aCard suit = leadCard suit))
	ifTrue: [self playNonTrumpCard: aCard]
		ifFalse: [self error: 'You must play the lead suit or a trump card!!']].
	
	aHand removeCard: aCard.
! !

!SetbackTrick methodsFor: 'play-logic' stamp: 'TerryTata 6/29/2013 13:58'!
playNonTrumpCard: aCard
"plays a trump card in the current trick."
aCard suit = winningCard suit
ifTrue: [ self playNormalCard: aCard. ]
ifFalse: [ cards add: aCard.]! !

!SetbackTrick methodsFor: 'play-logic' stamp: 'TerryTata 6/29/2013 12:37'!
playNormalCard: aCard
"plays a card in the current trick."
cards add: aCard.
aCard > winningCard 
ifTrue: [ winningCard:= aCard. ]! !

!SetbackTrick methodsFor: 'play-logic' stamp: 'TerryTata 6/29/2013 12:41'!
playTrumpCard: aCard
"plays a trump card in the current trick."
winningCard suit = trumpSuit
ifTrue: [ self playNormalCard: aCard. ]
ifFalse: [ cards add: aCard. winningCard:= aCard ]! !

!SetbackTrick methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:15'!
trumpSuit
^trumpSuit copy! !

!SetbackTrick methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:15'!
winningCard
^winningCard copy! !

Object subclass: #WinnerStrategy
	instanceVariableNames: 'pointsToWin winningMargin bidToWin playerConfig'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

WinnerStrategy class
	instanceVariableNames: 'testConfig'!

!WinnerStrategy commentStamp: '<historical>' prior: 0!
A WinnerStrategy is xxxxxxxxx.

Instance Variables
	bidToWin:		<Object>
	pointsToWin:		<Object>
	winningMargin:		<Object>

bidToWin
	- xxxxx

pointsToWin
	- xxxxx

winningMargin
	- xxxxx
!

WinnerStrategy class
	instanceVariableNames: 'testConfig'!

WinnerStrategy subclass: #FirstToElevenWinStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!FirstToElevenWinStrategy commentStamp: '<historical>' prior: 0!
A FirstToElevenWinStrategy is xxxxxxxxx.

Instance Variables
!

!FirstToElevenWinStrategy methodsFor: 'as yet unclassified' stamp: 'RameshKolavenu 7/23/2013 01:01'!
initialize
	FirstToElevenWinStrategy testConfig isNil 
		ifTrue:[pointsToWin:= 11]
		ifFalse:[pointsToWin:= 2].
winningMargin:= 0. 
bidToWin:= false.! !

!FirstToElevenWinStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 10:57'!
winnerWithTeams: players bidder: aBidder
	"when we play with teams, it will be two teams of two"
	| teams highestPoints winningTeam tie |
	teams := self extractTeams: players.
	highestPoints := 0.
	teams
		do: [ :team | 
			team pointsEarned = highestPoints
				ifTrue: [ tie := true ].
			team pointsEarned > highestPoints
				ifTrue: [ 
					highestPoints := team pointsEarned.
					winningTeam := team.
					tie := false ] ].
	highestPoints >= pointsToWin & tie not
		ifTrue: [ 
			winningTeam incrementGamesWon.
			^ winningTeam ].
		
	^nil! !

!FirstToElevenWinStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 14:30'!
winnerWithoutTeams: players bidder: aBidder
	| highestPoints winningPlayer tie |
	highestPoints := 0.	"make sure the threshhold for winning has been reached"
	players
		do: [ :player | 
			player pointsEarned = highestPoints
				ifTrue: [ tie := true ].
			player pointsEarned > highestPoints
				ifTrue: [ 
					highestPoints := player pointsEarned.
					winningPlayer := player.
					tie := false ] ].	"we don't allow ties"
	highestPoints >= pointsToWin & tie not
		ifTrue: [ 
			winningPlayer incrementGamesWon.
			^ winningPlayer ].
	^ nil! !

WinnerStrategy subclass: #StandardWinStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!StandardWinStrategy commentStamp: '<historical>' prior: 0!
A StandardWinStrategy is xxxxxxxxx.

Instance Variables
!

!StandardWinStrategy methodsFor: 'as yet unclassified' stamp: 'RameshKolavenu 7/23/2013 01:02'!
initialize
	StandardWinStrategy testConfig isNil 
		ifTrue:[pointsToWin:= 11]
		ifFalse:[pointsToWin:= 2].
	winningMargin:= 2. 
	bidToWin:= true.! !

!StandardWinStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 10:57'!
winnerWithTeams: players bidder: aBidder
"when we play with teams, it will be two teams of two"
	"when we play with teams, it will be two teams of two"
	| teams highestPoints |
	teams := self extractTeams: players.
	highestPoints := 0.
		aBidder currentTeam pointsEarned >= pointsToWin
		ifFalse: [ ^ nil ].
	teams
		do: [ :team | 
			team = aBidder currentTeam
				ifFalse: [highestPoints:= team pointsEarned]].
	aBidder currentTeam pointsEarned - highestPoints > winningMargin
		ifTrue: [ 
			aBidder currentTeam incrementGamesWon.
			^ aBidder currentTeam ].
		
	^nil! !

!StandardWinStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/11/2013 12:13'!
winnerWithoutTeams: players bidder: aBidder
	| highestPoints |
	highestPoints := 0.	"make sure the threshhold for winning has been reached"
	aBidder pointsEarned >= pointsToWin
		ifFalse: [ ^ nil ].
	players
		do: [ :player | 
			aBidder = player
				ifFalse: [ 
					player pointsEarned > highestPoints
						ifTrue: [ highestPoints := player pointsEarned ] ] ].	"make sure the gap is sufficient between the next highest player"
	aBidder pointsEarned - highestPoints > winningMargin
		ifTrue: [ 
			aBidder incrementGamesWon.
			^ aBidder ].
	^ nil! !

!WinnerStrategy class methodsFor: 'as yet unclassified' stamp: 'RameshKolavenu 7/23/2013 00:22'!
resetTestConfig
	testConfig:= nil.! !

!WinnerStrategy class methodsFor: 'as yet unclassified' stamp: 'RameshKolavenu 7/23/2013 00:14'!
setTestConfig
	testConfig:= true.! !

!WinnerStrategy class methodsFor: 'as yet unclassified' stamp: 'RameshKolavenu 7/23/2013 00:14'!
testConfig
	^testConfig.! !

!WinnerStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/11/2013 11:46'!
doWeHaveAWinner: players bidder: aBidder playerConfig: anObject
playerConfig:= anObject.

playerConfig onTeams
ifTrue: [ ^self winnerWithTeams: players copy bidder: aBidder. ]
ifFalse: [ ^self winnerWithoutTeams: players copy bidder: aBidder. ].! !

!WinnerStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 10:57'!
extractTeams: players
"setback operates with two teams. player order is determined by teams."
|ll|
ll:= LinkedList new.
ll add: players first currentTeam.
ll add: players last currentTeam.
^ll! !

!WinnerStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 10:56'!
winnerWithTeams: players bidder: aBidder
^self subclassResponsibility! !

!WinnerStrategy methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/11/2013 12:14'!
winnerWithoutTeams: players bidder: aBidder
^self subclassResponsibility! !

Model subclass: #SetbackPlayer
	instanceVariableNames: 'name hand pointsEarned gamePointsWon cardsWon gamesWon playerId currentTeam currentBid hasCokeHand'
	classVariableNames: 'playerIdCounter'
	poolDictionaries: ''
	category: 'CS528-SetbackProject'!

!SetbackPlayer commentStamp: '<historical>' prior: 0!
I am a player of a setback game. I can be dealt cards, select a card to play, and win tricks. I keep a tally of the points I earned this game, the cards I have won this hand and the current cards in my hand.!

!SetbackPlayer class methodsFor: 'accessing' stamp: 'TerryTata 6/30/2013 13:39'!
name: aName
(playerIdCounter = nil) ifTrue: [ playerIdCounter:= 0 ].
^self new name: aName! !

!SetbackPlayer class methodsFor: 'accessing' stamp: 'TerryTata 7/16/2013 18:55'!
resetPlayerId
playerIdCounter:= 0.! !

!SetbackPlayer methodsFor: 'comparing' stamp: 'MehrdadBiglari 7/17/2013 21:59'!
= aPlayer
aPlayer ifNil: [ ^false ].
^self playerId = aPlayer playerId! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 7/6/2013 10:12'!
addToGamePoints: aValue
gamePointsWon:= gamePointsWon + aValue.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 7/6/2013 17:50'!
addToPointsEarned: somePoints
pointsEarned:= pointsEarned + somePoints.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 6/30/2013 16:36'!
beDealtCard: aCard
hand addCard: aCard! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:40'!
cardsWon
^cardsWon copy! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 7/2/2013 21:29'!
clearCardsWon
cardsWon removeAll.! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'MehrdadBiglari 7/17/2013 21:23'!
currentBid
^currentBid! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'TerryTata 11/29/1999 16:49'!
currentTeam
^currentTeam! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'TerryTata 7/6/2013 10:17'!
gamePointsWon
^gamePointsWon copy! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'TerryTata 7/7/2013 18:56'!
gamesWon
^gamesWon copy! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:34'!
hand
^hand copy! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 7/3/2013 21:18'!
hand: aHand
"for testing"
hand:= aHand.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 7/21/2013 11:19'!
hasCokeHand
^hand isCokeHand! !

!SetbackPlayer methodsFor: 'comparing' stamp: 'TerryTata 6/30/2013 12:58'!
hash
^playerId hash.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 7/7/2013 20:32'!
incrementGamesWon
gamesWon:= gamesWon + 1.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 11/29/1999 16:18'!
joinTeam: aTeam
currentTeam isNil
ifFalse: [ self error: 'You are already on a team!! You must leave your current team before joining a new one!!' ].
currentTeam:= aTeam.
aTeam addPlayer: self.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 11/29/1999 16:19'!
leaveTeam
currentTeam isNil
ifTrue: [ self error: 'You are not on a team!!' ].
currentTeam removePlayer: self.
currentTeam:= nil.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'MehrdadBiglari 7/17/2013 21:22'!
makeBid: aBid
(CardUtility possibleBids includesKey: aBid)
ifFalse: [ self error: 'This bid is not valid!!' ].
currentBid:= aBid.
self changed: aBid! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:39'!
name
^name copy! !

!SetbackPlayer methodsFor: 'initialize-release' stamp: 'TerryTata 7/7/2013 19:07'!
name: aName
name:= aName.
hand:= SetbackPlayerHand new.
pointsEarned:= 0.
gamePointsWon:= 0.
gamesWon:= 0.
cardsWon:= Set new.
playerIdCounter:= playerIdCounter+ 1.
playerId:= playerIdCounter.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'MehrdadBiglari 7/17/2013 23:30'!
playCard: aCard
self changed: aCard.
^aCard! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'TerryTata 6/30/2013 12:58'!
playerId
^playerId copy! !

!SetbackPlayer methodsFor: 'accessing' stamp: 'TerryTata 6/29/2013 21:39'!
pointsEarned
^pointsEarned copy! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 7/11/2013 11:39'!
resetForNewGame
self resetForNewGameHand.
pointsEarned:= 0.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'MehrdadBiglari 7/17/2013 21:23'!
resetForNewGameHand
gamePointsWon:= 0.
cardsWon:= cardsWon removeAll.
currentBid:= nil.! !

!SetbackPlayer methodsFor: 'actions' stamp: 'TerryTata 6/29/2013 21:43'!
winTrick: someCards
cardsWon addAll: someCards! !

TestCase subclass: #SetbackTestCase
	instanceVariableNames: 'sortedDeck oneHeart oneDiamond oneClub oneSpade player1 player2 player3 player4 ll'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject-Tests'!

SetbackTestCase subclass: #CardHandTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject-Tests'!

!CardHandTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 12:11'!
testHandAdditionCapacity
|hand deck|
deck:= (FrenchDeck createStandardDeck).
hand:= (SetbackPlayerHand addCard: deck dealNext).
self assert: (hand cards size) = 1.
(hand addCard: deck dealNext).
self assert: (hand cards size) = 2.
(hand addCard: deck dealNext).
self assert: (hand cards size) = 3.
(hand addCard: deck dealNext).
self assert: (hand cards size) = 4.
(hand addCard: deck dealNext).
self assert: (hand cards size) = 5.
(hand addCard: deck dealNext).
self assert: (hand cards size) = 6.
self should: [ (hand addCard: deck dealNext). ] raise: TestResult error.
! !

!CardHandTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 12:11'!
testHandRemovalCapacity
|hand card|
card:= (Card value: #two suit: #hearts).
hand:= (SetbackPlayerHand addCard: card).
self assert: (hand capacity) = 6.
self assert: (hand cards size) = 1.
(hand removeCard: card).
self should: [ (hand removeCard: card). ] raise: TestResult error.
! !

!CardHandTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 12:11'!
testHandUniqueness
|hand card|
card:= (Card value: #two suit: #hearts).
hand:= (SetbackPlayerHand addCard: card).
self assert: (hand capacity) = 6.
self assert: (hand cards size) = 1.
(SetbackPlayerHand addCard: card).
self assert: (hand cards size) = 1.! !

SetbackTestCase subclass: #CardTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject-Tests'!

!CardTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 11:07'!
testCardUniqueness
|card|
card:= (Card value: #two suit: #hearts).
self should: [ card value: #two suit: #hearts ] raise: TestResult error.! !

!CardTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/28/2013 15:29'!
testInvalidCardCreation
self should: [(Card value: #two unit: #two)] raise: TestResult error.
self should: [(Card value: #hearts unit: #hearts)] raise: TestResult error.! !

!CardTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 11:07'!
testSimpleCardCreation
self assert: ((Card value: #two suit: #hearts) value) = #two.
self assert: ((Card value: #two suit: #hearts) suit) = #hearts.! !

SetbackTestCase subclass: #DeckTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject-Tests'!

!DeckTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 07:58'!
testDeckCreation
self assert: (FrenchDeck new cards = nil).
self assert: (FrenchDeck createStandardDeck cards size = 52).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #ace suit: #hearts)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #king suit: #diamonds)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #queen suit: #spades)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #jack suit: #clubs)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #ten suit: #hearts)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #nine suit: #diamonds)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #eight suit: #spades)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #seven suit: #clubs)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #six suit: #clubs)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #five suit: #hearts)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #four suit: #diamonds)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #three suit: #spades)).
self assert: (FrenchDeck createStandardDeck cards includes: (Card value: #two suit: #clubs)).! !

!DeckTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 07:58'!
testDeckDealing
|deck|
deck:= FrenchDeck createStandardDeck.
1 to: 52 do: [ :i | deck dealNext ].
self should: [deck dealNext] raise: TestResult error.! !

!DeckTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/19/2013 07:48'!
testDeckSingleton
|deck|
deck:= FrenchDeck createStandardDeck.
self assert: deck cards size = 52.
self assert: deck currentIndex = 0.! !

SetbackTestCase subclass: #SetbackGameTest
	instanceVariableNames: 'game hand team1 team2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject-Tests'!

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:39'!
setUpWithDifferentWinStrategy
ll:= LinkedList new.
(ll add: player1).
(ll add: player2).
(ll add: player3).
game:= SetbackGame config: (ThreePlayerStrategy new) winner: (FirstToElevenWinStrategy new) order: (SetbackPhaseOrder new).
game addPlayer: player1.
game addPlayer: player2.
game addPlayer: player3.
hand:= game startGame.
hand startNextPhase.
hand startNextPhase.
player2 makeBid: #two.
player3 makeBid: #pass.
player1 makeBid: #two.
"player one wins the bid"! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:45'!
setUpWithNoJunkPoints
|phaseOrder|
ll:= LinkedList new.
(ll add: player1).
(ll add: player2).
(ll add: player3).
phaseOrder:= SetbackPhaseOrder new.
phaseOrder scorePhase: #NoGarbagePointsScorePhase.
game:= SetbackGame config: (ThreePlayerStrategy new) winner: (StandardWinStrategy  new) order: phaseOrder.
game addPlayer: player1.
game addPlayer: player2.
game addPlayer: player3.
hand:= game startGame.
hand startNextPhase.
hand startNextPhase.
player2 makeBid: #two.
player3 makeBid: #pass.
player1 makeBid: #two.
"player one wins the bid"! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:36'!
standardDealAndBidPhases
ll:= LinkedList new.
(ll add: player1).
(ll add: player2).
(ll add: player3).
game:= SetbackGame setUpStandardGame.
game addPlayer: player1.
game addPlayer: player2.
game addPlayer: player3.
hand:= game startGame.
hand startNextPhase.
hand startNextPhase.
player2 makeBid: #two.
player3 makeBid: #pass.
player1 makeBid: #two.
"player one wins the bid"! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:36'!
standardTrickAndScorePhases
player1 hand: oneClub copy.
player2 hand: oneSpade copy.
player3 hand: oneDiamond copy.
hand startNextPhase.
"hearts as trump - player 2 wins trick"
player1 playCard: (sortedDeck at: 18).
player2 playCard: (sortedDeck at: 22).
player3 playCard: (sortedDeck at: 10).
"trick 2 - player 2 wins"
player1 playCard: (sortedDeck at: 13).
"we shouldn't be listening to player 1, only player 2 who is next to throw"
self assert: (player1 hand cards size) =  5.
player2 playCard: (sortedDeck at: 25).
player3 playCard: (sortedDeck at: 41).
player1 playCard: (sortedDeck at: 17).
"trick 3 - player 2 wins"
player2 playCard: (sortedDeck at: 19).
player3 playCard: (sortedDeck at: 11).
player1 playCard: (sortedDeck at: 15).
"trick 4 - player 3 wins"
player2 playCard: (sortedDeck at: 20).
player3 playCard: (sortedDeck at: 8).
player1 playCard: (sortedDeck at: 16).
"trick 5 - player 3 wins"
player3 playCard: (sortedDeck at: 12).
player1 playCard: (sortedDeck at: 13).
player2 playCard: (sortedDeck at: 21).
"trick 6 - player 3 wins"
player3 playCard: (sortedDeck at: 50).
player1 playCard: (sortedDeck at: 14).
player2 playCard: (sortedDeck at: 23).
hand startNextPhase.
hand startNextPhase.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 16:02'!
teamDealAndBidPhases
team1:= SetbackPlayerTeam new.
team2:= SetbackPlayerTeam new.
player1 joinTeam: team1.
player2 joinTeam: team2.
player3 joinTeam: team1.
player4 joinTeam: team2.
game:= SetbackGame setUpStandardTeamGame.
game addPlayer: player1.
game addPlayer: player2.
game addPlayer: player3.
game addPlayer: player4.
hand:= game startGame.
hand startNextPhase.
hand startNextPhase.
player2 makeBid: #two.
player3 makeBid: #pass.
player4 makeBid: #pass.
player1 makeBid: #two.
"player1 wins the bid"! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 09:39'!
teamTrickAndScorePhases
player1 hand: oneClub.
player2 hand: oneSpade.
player3 hand: oneDiamond.
player4 hand: oneHeart.
hand startNextPhase.
"hearts as trump - player 2 wins trick"
player1 playCard: (sortedDeck at: 18).
player2 playCard: (sortedDeck at: 22).
player3 playCard: (sortedDeck at: 10).
player4 playCard: (sortedDeck at: 2).
"trick 2 - player 2 wins"
player1 playCard: (sortedDeck at: 13).
"we shouldn't be listening to player 1, only player 2 who is next to throw"
self assert: (player1 hand cards size) =  5.
player2 playCard: (sortedDeck at: 25).
player3 playCard: (sortedDeck at: 41).
player4 playCard: (sortedDeck at: 5).
player1 playCard: (sortedDeck at: 17).
"trick 3 - player 2 wins"
player2 playCard: (sortedDeck at: 19).
player3 playCard: (sortedDeck at: 11).
player4 playCard: (sortedDeck at: 3).
player1 playCard: (sortedDeck at: 15).
"trick 4 - player 3 wins"
player2 playCard: (sortedDeck at: 20).
player3 playCard: (sortedDeck at: 8).
player4 playCard: (sortedDeck at: 4).
player1 playCard: (sortedDeck at: 16).
"trick 5 - player 4 wins"
player3 playCard: (sortedDeck at: 12).
player4 playCard: (sortedDeck at: 28).
player1 playCard: (sortedDeck at: 13).
player2 playCard: (sortedDeck at: 21).
"trick 6 - player 3 wins"
player4 playCard: (sortedDeck at: 1).
player1 playCard: (sortedDeck at: 14).
player2 playCard: (sortedDeck at: 23).
player3 playCard: (sortedDeck at: 50).
hand startNextPhase.
hand startNextPhase.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 15:09'!
testBasicTeamTrickPhase
self teamDealAndBidPhases.
player1 hand: oneClub.
player2 hand: oneSpade.
player3 hand: oneDiamond.
player4 hand: oneHeart.
hand startNextPhase.
"hearts as trump"
player1 playCard: (sortedDeck at: 18).
self assert: (player1 hand cards size) =  5.
"can't play diamonds"
self should: [player2 playCard: (sortedDeck at: 21)] raise: TestResult error.
player2 playCard: (sortedDeck at: 22).
self assert: (player2 hand cards size) =  5.
player3 playCard: (sortedDeck at: 10).
self assert: (player3 hand cards size) =  5.
player4 playCard: (sortedDeck at: 2).
self assert: (player4 hand cards size) =  5.
"the trick is over according to our player configuration"
self assert: (player2 cardsWon size) = 4.
! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:36'!
testBasicTrickPhase
self standardDealAndBidPhases.
player1 hand: oneClub.
player2 hand: oneSpade.
player3 hand: oneDiamond.
hand startNextPhase.
"hearts as trump"
player1 playCard: (sortedDeck at: 18).
self assert: (player1 hand cards size) =  5.
"can't play diamonds"
self should: [player2 playCard: (sortedDeck at: 21)] raise: TestResult error.
player2 playCard: (sortedDeck at: 22).
self assert: (player2 hand cards size) =  5.
player3 playCard: (sortedDeck at: 10).
self assert: (player3 hand cards size) =  5.
"the trick is over according to our player configuration"
self assert: (player2 cardsWon size) = 3.
! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/21/2013 11:37'!
testCokeHand
self standardDealAndBidPhases.
player1 hand: oneClub.
player2 hand: oneSpade.
player3 hand: oneDiamond.
player4 hand: oneHeart.

self assert: (player1 hasCokeHand) = true.
self assert: (player2 hasCokeHand) = false.
self assert: (player3 hasCokeHand) = false.
self assert: (player4 hasCokeHand) = false.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 14:57'!
testDealAndBidWithTeams
| team1 team2|
team1:= SetbackPlayerTeam new.
team2:= SetbackPlayerTeam new.
player1 joinTeam: team1.
player2 joinTeam: team2.
player3 joinTeam: team1.
player4 joinTeam: team2.
game:= SetbackGame setUpStandardTeamGame.
game addPlayer: player1.
game addPlayer: player2.
game addPlayer: player3.
game addPlayer: player4.
hand:= game startGame.
hand startNextPhase.
hand startNextPhase.
player2 makeBid: #pass.
player3 makeBid: #pass.
player4 makeBid: #pass.
player1 makeBid: #pass.
self assert: hand bidder = player1. 
self assert: hand currentBid = #two. 
self assert: player1 hand cards size = 6.
self assert: player2 hand cards size = 6.
self assert: player3 hand cards size = 6.
self assert: player4 hand cards size = 6.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 09:30'!
testFullTeamTrickPhase
self teamDealAndBidPhases.
player1 hand: oneClub.
player2 hand: oneSpade.
player3 hand: oneDiamond.
player4 hand: oneHeart.
hand startNextPhase.
"hearts as trump - player 2 wins trick"
player1 playCard: (sortedDeck at: 18).
player2 playCard: (sortedDeck at: 22).
player3 playCard: (sortedDeck at: 10).
player4 playCard: (sortedDeck at: 2).
"trick 2 - player 2 wins"
player1 playCard: (sortedDeck at: 13).
"we shouldn't be listening to player 1, only player 2 who is next to throw"
self assert: (player1 hand cards size) =  5.
player2 playCard: (sortedDeck at: 25).
player3 playCard: (sortedDeck at: 41).
player4 playCard: (sortedDeck at: 5).
player1 playCard: (sortedDeck at: 17).
"trick 3 - player 2 wins"
player2 playCard: (sortedDeck at: 19).
player3 playCard: (sortedDeck at: 11).
player4 playCard: (sortedDeck at: 3).
player1 playCard: (sortedDeck at: 15).
"trick 4 - player 3 wins"
player2 playCard: (sortedDeck at: 20).
player3 playCard: (sortedDeck at: 8).
player4 playCard: (sortedDeck at: 4).
player1 playCard: (sortedDeck at: 16).
"trick 5 - player 4 wins"
player3 playCard: (sortedDeck at: 12).
player4 playCard: (sortedDeck at: 28).
player1 playCard: (sortedDeck at: 13).
player2 playCard: (sortedDeck at: 21).
"trick 6 - player 3 wins"
player4 playCard: (sortedDeck at: 1).
player1 playCard: (sortedDeck at: 14).
player2 playCard: (sortedDeck at: 23).
player3 playCard: (sortedDeck at: 50).

self assert: (player1 hand cards size) = 0.
self assert: (player2 hand cards size) = 0.
self assert: (player3 hand cards size) = 0.
self assert: (player4 hand cards size) = 0.
self assert: (player1 cardsWon size) = 0.
self assert: (player2 cardsWon size) = 12.
self assert: (player3 cardsWon size) = 8.
self assert: (player4 cardsWon size) = 4.
self assert: (hand trump) = #hearts.
hand startNextPhase.
hand startNextPhase.
self assert: player1 pointsEarned + player3 pointsEarned = -2.
self assert: player2 pointsEarned + player4  pointsEarned = 2.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:36'!
testFullTrickPhase
self standardDealAndBidPhases.
player1 hand: oneClub.
player2 hand: oneSpade.
player3 hand: oneDiamond.
hand startNextPhase.
"hearts as trump - player 2 wins trick"
player1 playCard: (sortedDeck at: 18).
player2 playCard: (sortedDeck at: 22).
player3 playCard: (sortedDeck at: 10).
"trick 2 - player 2 wins"
player1 playCard: (sortedDeck at: 13).
"we shouldn't be listening to player 1, only player 2 who is next to throw"
self assert: (player1 hand cards size) =  5.
player2 playCard: (sortedDeck at: 25).
player3 playCard: (sortedDeck at: 41).
player1 playCard: (sortedDeck at: 17).
"trick 3 - player 2 wins"
player2 playCard: (sortedDeck at: 19).
player3 playCard: (sortedDeck at: 11).
player1 playCard: (sortedDeck at: 15).
"trick 4 - player 3 wins"
player2 playCard: (sortedDeck at: 20).
player3 playCard: (sortedDeck at: 8).
player1 playCard: (sortedDeck at: 16).
"trick 5 - player 3 wins"
player3 playCard: (sortedDeck at: 12).
player1 playCard: (sortedDeck at: 13).
player2 playCard: (sortedDeck at: 21).
"trick 6 - player 3 wins"
player3 playCard: (sortedDeck at: 50).
player1 playCard: (sortedDeck at: 14).
player2 playCard: (sortedDeck at: 23).

self assert: (player1 hand cards size) = 0.
self assert: (player2 hand cards size) = 0.
self assert: (player3 hand cards size) = 0.
self assert: (player1 cardsWon size) = 0.
self assert: (player2 cardsWon size) = 9.
self assert: (player3 cardsWon size) = 9.
self assert: (hand trump) = #hearts.
hand startNextPhase.
hand startNextPhase.
self assert: player1 pointsEarned = -2.
self assert: player2 pointsEarned = 1.
self assert: player3  pointsEarned = 2.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/23/2013 19:45'!
testFullTrickPhaseNoJunk
self setUpWithNoJunkPoints.
player1 hand: oneClub.
player2 hand: oneSpade.
player3 hand: oneDiamond.
hand startNextPhase.
"hearts as trump - player 2 wins trick"
player1 playCard: (sortedDeck at: 18).
player2 playCard: (sortedDeck at: 22).
player3 playCard: (sortedDeck at: 10).
"trick 2 - player 2 wins"
player1 playCard: (sortedDeck at: 13).
"we shouldn't be listening to player 1, only player 2 who is next to throw"
self assert: (player1 hand cards size) =  5.
player2 playCard: (sortedDeck at: 25).
player3 playCard: (sortedDeck at: 41).
player1 playCard: (sortedDeck at: 17).
"trick 3 - player 2 wins"
player2 playCard: (sortedDeck at: 19).
player3 playCard: (sortedDeck at: 11).
player1 playCard: (sortedDeck at: 15).
"trick 4 - player 3 wins"
player2 playCard: (sortedDeck at: 20).
player3 playCard: (sortedDeck at: 8).
player1 playCard: (sortedDeck at: 16).
"trick 5 - player 3 wins"
player3 playCard: (sortedDeck at: 12).
player1 playCard: (sortedDeck at: 13).
player2 playCard: (sortedDeck at: 21).
"trick 6 - player 3 wins"
player3 playCard: (sortedDeck at: 50).
player1 playCard: (sortedDeck at: 14).
player2 playCard: (sortedDeck at: 23).

self assert: (player1 hand cards size) = 0.
self assert: (player2 hand cards size) = 0.
self assert: (player3 hand cards size) = 0.
self assert: (player1 cardsWon size) = 0.
self assert: (player2 cardsWon size) = 9.
self assert: (player3 cardsWon size) = 9.
self assert: (hand trump) = #hearts.
hand startNextPhase.
hand startNextPhase.
self assert: player1 pointsEarned = -2.
self assert: player2 pointsEarned = 0.
self assert: player3  pointsEarned = 0.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 17:09'!
testGameCreation
|game|
player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Joe'.
player3:= SetbackPlayer name: 'Adam'.
game:= SetbackGame setUpStandardGame.
game addPlayer: player1.
game addPlayer: player2.
game addPlayer: player3.
game startGame.

self assert: game started = true.
! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 14:52'!
testGameCreationTeams
| team1 team2|
team1:= SetbackPlayerTeam new.
team2:= SetbackPlayerTeam new.
player1 joinTeam: team1.
player2 joinTeam: team2.
player3 joinTeam: team1.
player4 joinTeam: team2.
game:= SetbackGame setUpStandardTeamGame.
game addPlayer: player1.
game addPlayer: player2.
game addPlayer: player3.
game addPlayer: player4.
game startGame.

self assert: game started = true.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 18:18'!
testGameStartAndBid
player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Joe'.
player3:= SetbackPlayer name: 'Adam'.
ll:= LinkedList new.
(ll add: player1).
(ll add: player2).
(ll add: player3).
hand:= SetbackGameHand players: ll playerConfig: nil winnerStrat: nil phaseOrder: (StandardPhaseOrder new).
hand startNextPhase.
hand startNextPhase.
player2 makeBid: #pass.
player3 makeBid: #pass.
player1 makeBid: #pass.
self assert: hand bidder = player1. 
self assert: hand currentBid = #two. ! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 18:18'!
testGameStartAndBid2
player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Joe'.
player3:= SetbackPlayer name: 'Adam'.
ll:= LinkedList new.
(ll add: player1).
(ll add: player2).
(ll add: player3).
hand:= SetbackGameHand players: ll playerConfig: nil winnerStrat: nil phaseOrder: (StandardPhaseOrder new).
hand startNextPhase.
hand startNextPhase.
player2 makeBid: #two.
player3 makeBid: #three.
player1 makeBid: #two.
self assert: hand bidder = player3. 
self assert: hand currentBid = #three. ! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 18:18'!
testGameStartAndDealerStealsBid
player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Joe'.
player3:= SetbackPlayer name: 'Adam'.
ll:= LinkedList new.
(ll add: player1).
(ll add: player2).
(ll add: player3).
hand:= SetbackGameHand players: ll playerConfig: nil winnerStrat: nil phaseOrder: (StandardPhaseOrder new).
hand startNextPhase.
hand startNextPhase.
player2 makeBid: #two.
player3 makeBid: #pass.
player1 makeBid: #two.
self assert: hand bidder = player1. 
self assert: hand currentBid = #two. ! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:36'!
testGameWinner
self standardDealAndBidPhases.
player1 addToPointsEarned: 15.
self standardTrickAndScorePhases.
self assert: (player1 gamesWon) = 1.
self assert: (player2 gamesWon) = 0.
self assert: (player3 gamesWon) = 0.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:43'!
testGameWinnerAlt
self setUpWithDifferentWinStrategy.
player2 addToPointsEarned: 9.
player3 addToPointsEarned: 9.
self standardTrickAndScorePhases.
self assert: (player1 gamesWon) = 0.
self assert: (player2 gamesWon) = 0.
self assert: (player3 gamesWon) = 1.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:45'!
testGameWinnerAlt2
self setUpWithDifferentWinStrategy.
player2 addToPointsEarned: 10.
player3 addToPointsEarned: 9.
self standardTrickAndScorePhases.
self assert: (player1 gamesWon) = 0.
self assert: (player2 gamesWon) = 0.
self assert: (player3 gamesWon) = 0.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:36'!
testGameWinnerMarginNotLargeEnough
self standardDealAndBidPhases.
player1 addToPointsEarned: 15.
player2 addToPointsEarned: 10.
self standardTrickAndScorePhases.
self assert: (player1 gamesWon) = 0.
self assert: (player2 gamesWon) = 0.
self assert: (player3 gamesWon) = 0.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 09:38'!
testGameWinnerTeams
self teamDealAndBidPhases.
player1 addToPointsEarned: 15.
self teamTrickAndScorePhases.
self assert: (player1 gamesWon) = 1.
self assert: (player2 gamesWon) = 0.
self assert: (player3 gamesWon) = 1.
self assert: (player4 gamesWon) = 0.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/13/2013 12:07'!
testGameWinnerTeamsMarginNotLargeEnough
self teamDealAndBidPhases.
player3 addToPointsEarned: 15.
player4 addToPointsEarned: 10.
self teamTrickAndScorePhases.
self assert: (player1 gamesWon) = 0.
self assert: (player2 gamesWon) = 0.
self assert: (player3 gamesWon) = 0.
self assert: (player4 gamesWon) = 0.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 17:09'!
testInvalidGameCreation
|player4 game|
player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Joe'.
player3:= SetbackPlayer name: 'Adam'.
player4:= SetbackPlayer name: 'Homer'.
game:= SetbackGame setUpStandardGame.
game addPlayer: player1.
self should: [ game startGame ] raise: TestResult error.
game addPlayer: player2.
game addPlayer: player3.
self should: [ game addPlayer: player4. ] raise: TestResult error.
self should: [ game startNewHand ] raise: TestResult error.
game startGame.
self should: [ game playerConfigStrategy: ThreePlayerStrategy new. ] raise: TestResult error.
self should: [ game winnerStrategy: StandardWinStrategy new. ] raise: TestResult error.
! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/7/2013 18:18'!
testPlayerDealing
player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Joe'.
ll:= LinkedList new.
(ll add: player1).
(ll add: player2).
hand:= SetbackGameHand players: ll playerConfig: nil winnerStrat: nil phaseOrder: (StandardPhaseOrder new).
hand startNextPhase.
self assert: (player1 hand cards size) = 6.
self assert: (player2 hand cards size) = 6.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:36'!
testSubsequentGames
self standardDealAndBidPhases.
self standardTrickAndScorePhases.
hand:= game startNewHand.
"ensure that the new hand is independent of the old hand"
self assert: hand trump = nil.
self assert: hand bidder = nil.
self assert: hand currentBid = nil.
self assert: hand currentPhase = nil.
self assert: hand players removeFirst pointsEarned ~= 0.! !

!SetbackGameTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 13:36'!
testSubsequentGamesErrors
self standardDealAndBidPhases.
self standardTrickAndScorePhases.
self should: [ game addPlayer: player1] raise: TestResult error.
self should: [ game phaseOrder: player1] raise: TestResult error.
self should: [ game playerConfigStrategy: player1] raise: TestResult error.
self should: [ game winnerStrategy: player1] raise: TestResult error.
self should: [ game startGame] raise: TestResult error.! !

SetbackTestCase subclass: #SetbackPlayerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject-Tests'!

!SetbackPlayerTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 09:32'!
testPlayerCreation
self assert: (SetbackPlayer name: 'Terry') name = 'Terry'.
self assert: (SetbackPlayer name: 'Terry') hand cards size = 0.
self assert: (SetbackPlayer name: 'Terry') pointsEarned = 0.
self assert: (SetbackPlayer name: 'Terry') cardsWon size = 0.! !

!SetbackPlayerTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 16:24'!
testPlayerEquality
self assert: (SetbackPlayer name: 'Terry') ~= (SetbackPlayer name: 'Terry')! !

!SetbackPlayerTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 11/29/1999 16:52'!
testPlayerTeamAddition
| player4 team1 team2|
player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Ramesh'.
player3:= SetbackPlayer name: 'Mehrdad'.
player4:= SetbackPlayer name: 'Adam'.
team1:= SetbackPlayerTeam new.
team2:= SetbackPlayerTeam new.
player1 joinTeam: team1.
player2 joinTeam: team2.
player3 joinTeam: team2.
player4 joinTeam: team1.
self assert: team1 players size = 2.
self assert: team2 players size = 2.
self assert: player1 currentTeam = player4 currentTeam.
self assert: player2 currentTeam = player3 currentTeam.
! !

!SetbackPlayerTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 11/29/1999 16:55'!
testPlayerTeamRemoval
| player4 team1 team2|
player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Ramesh'.
player3:= SetbackPlayer name: 'Mehrdad'.
player4:= SetbackPlayer name: 'Adam'.
team1:= SetbackPlayerTeam new.
team2:= SetbackPlayerTeam new.
player1 joinTeam: team1.
player2 joinTeam: team2.
player3 joinTeam: team2.
player4 joinTeam: team1.
player1 leaveTeam.
player2 leaveTeam.
self assert: team1 players size = 1.
self assert: team2 players size = 1.
self assert: player1 currentTeam = player2 currentTeam.! !

!SetbackTestCase methodsFor: 'as yet unclassified' stamp: 'TerryTata 7/12/2013 15:18'!
setUp
sortedDeck:= FrenchDeck new populateFrenchDeck.

player1:= SetbackPlayer name: 'Terry'.
player2:= SetbackPlayer name: 'Joe'.
player3:= SetbackPlayer name: 'Adam'.
player4:= SetbackPlayer name: 'Bob'.

oneHeart:= SetbackPlayerHand addCard: (sortedDeck at: 1). "4 of diamonds"
oneHeart 
	addCard: (sortedDeck at: 2); " 4 of hearts"
	addCard: (sortedDeck at: 3); " 4 of clubs"
	addCard: (sortedDeck at: 4); " 4 of spades"
	addCard: (sortedDeck at: 5); "10 of diamonds"
	addCard: (sortedDeck at: 28). " king of spades"
	
oneDiamond:= SetbackPlayerHand addCard: (sortedDeck at: 50)."ace of hearts"
oneDiamond
	addCard: (sortedDeck at: 8); "10 of spades"
	addCard: (sortedDeck at: 41); "jack of diamonds"
	addCard: (sortedDeck at: 10); "2 of hearts"
	addCard: (sortedDeck at: 11); "2 of clubs"
	addCard: (sortedDeck at: 12). "2 of spades"
	
oneClub:= SetbackPlayerHand addCard: (sortedDeck at: 13). "5 of diamonds"
oneClub 
	addCard: (sortedDeck at: 14); "5 of hearts"
	addCard: (sortedDeck at: 15); "5 of clubs"
	addCard: (sortedDeck at: 16); "5 of spades"
	addCard: (sortedDeck at: 17); "8 of diamonds"
	addCard: (sortedDeck at: 18). "8 of hearts"
	
oneSpade:= SetbackPlayerHand addCard: (sortedDeck at: 19). "8 of clubs"
oneSpade 
	addCard: (sortedDeck at: 20); "8 of spades"
	addCard: (sortedDeck at: 21); "queen of diamonds"
	addCard: (sortedDeck at: 22); "queen of hearts"
	addCard: (sortedDeck at: 23); "queen of clubs"
	addCard: (sortedDeck at: 25). "king of diamonds"! !

SetbackTestCase subclass: #SetbackTrickTest
	instanceVariableNames: 'oneHeart oneClub oneDiamond oneSpade'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CS528-SetbackProject-Tests'!

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 12:11'!
setUp
super setUp.

oneHeart:= SetbackPlayerHand new.
oneHeart addCard: (sortedDeck at: 1);
	addCard: (sortedDeck at: 2);
	addCard: (sortedDeck at: 3);
	addCard: (sortedDeck at: 4);
	addCard: (sortedDeck at: 6);
	addCard: (sortedDeck at: 7).
	
oneDiamond:= SetbackPlayerHand new.
oneDiamond addCard: (sortedDeck at: 50);
	addCard: (sortedDeck at: 8);
	addCard: (sortedDeck at: 41);
	addCard: (sortedDeck at: 10);
	addCard: (sortedDeck at: 11);
	addCard: (sortedDeck at: 12).
	
oneClub:= SetbackPlayerHand new.
oneClub addCard: (sortedDeck at: 13);
	addCard: (sortedDeck at: 14);
	addCard: (sortedDeck at: 15);
	addCard: (sortedDeck at: 16);
	addCard: (sortedDeck at: 17);
	addCard: (sortedDeck at: 18).
	
oneSpade:= SetbackPlayerHand new.
oneSpade addCard: (sortedDeck at: 19);
	addCard: (sortedDeck at: 20);
	addCard: (sortedDeck at: 21);
	addCard: (sortedDeck at: 22);
	addCard: (sortedDeck at: 23);
	addCard: (sortedDeck at: 25).! !

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 21:17'!
testOpeningTrick
|trick|
"4 of hearts"
trick:= SetbackTrick playFirstCardOfNewGame: (sortedDeck at: 2) inHand: oneHeart.
"ace of hearts"
trick playCard: (sortedDeck at: 50) inHand: oneDiamond.
"5 of hearts"
trick playCard: (sortedDeck at: 14) inHand: oneClub.
"queen of hearts"
trick playCard: (sortedDeck at: 22) inHand: oneSpade.
self assert: trick leadCard suit = #hearts.
self assert: trick leadCard value = #four.
self assert: trick winningCard suit = #hearts.
self assert: trick winningCard value = #ace.
self assert: trick cards size = 4.
self assert: trick trumpSuit = #hearts.! !

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 21:17'!
testTrickCantFollowSuit
|trick|
trick:= SetbackTrick playFirstCardOfNewGame: (sortedDeck at: 1) inHand: oneHeart.
trick playCard:  (sortedDeck at: 41) inHand: oneDiamond.
trick playCard: (sortedDeck at: 10) inHand: oneDiamond.
self assert: (trick cards size) = 3.
self assert: (trick leadCard) = (sortedDeck at: 1).
self assert: (trick trumpSuit) = (sortedDeck at: 1) suit.
self assert: (trick winningCard) =  (sortedDeck at: 41).! !

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 21:17'!
testTrickCardNotInHand
|trick|
trick:= SetbackTrick playFirstCardOfNewGame: (sortedDeck at: 1) inHand: oneHeart.
trick playCard:  (sortedDeck at: 41) inHand: oneDiamond.
self should: [ trick playCard: (sortedDeck at: 52) inHand: oneDiamond. ] raise: TestResult error.! !

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 21:17'!
testTrickCreation
|trick|
trick:= SetbackTrick leadCard: (sortedDeck at: 50) inHand: oneDiamond. 
self assert: trick leadCard suit = #hearts.
self assert: trick leadCard value = #ace.
self assert: trick cards size = 1.! !

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 21:17'!
testTrickCreationNewGame
|trick|
trick:= SetbackTrick playFirstCardOfNewGame: (sortedDeck at: 50) inHand: oneDiamond.
self assert: trick leadCard suit = #hearts.
self assert: trick leadCard value = #ace.
self assert: trick cards size = 1.
self assert: trick trumpSuit = #hearts.! !

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 21:17'!
testTrickFailToFollowSuit
|trick|
trick:= SetbackTrick playFirstCardOfNewGame: (sortedDeck at: 1) inHand: oneHeart. 
self should: [ trick playCard: (sortedDeck at: 10) inHand: oneDiamond ] raise: TestResult error.! !

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/29/2013 21:17'!
testTrickTrumpIn
|trick|
"Change trump to hearts"
SetbackTrick playFirstCardOfNewGame: (sortedDeck at: 2) inHand: oneHeart.
"create a new trick and lead with 10 of clubs"
trick:= SetbackTrick leadCard: (sortedDeck at: 7) inHand: oneHeart.
"trump in with 5 of hearts"
trick playCard: (sortedDeck at: 14) inHand: oneClub.
"throw a club to follow suit"
trick playCard: (sortedDeck at: 19) inHand: oneSpade.
"throw another trump"
trick playCard: (sortedDeck at: 10) inHand: oneDiamond.
self assert: trick leadCard suit = #clubs.
self assert: trick leadCard value = #ten.
self assert: trick winningCard suit = #hearts.
self assert: trick winningCard value = #five.
self assert: trick cards size = 4.
self assert: trick trumpSuit = #hearts.! !

!SetbackTrickTest methodsFor: 'as yet unclassified' stamp: 'TerryTata 6/30/2013 12:11'!
testTrickUniqueCards
|trick hand|
trick:= SetbackTrick playFirstCardOfNewGame: (sortedDeck at: 50) inHand: oneDiamond. 
hand:= SetbackPlayerHand addCard: (sortedDeck at: 50).
self assert: (trick playCard: (sortedDeck at: 50) inHand: hand) cards size = 1.! !
